@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: yasm.c 1978 2007-09-21 16:59:14Z peter $");
+/*@unused@*/ RCSID("$Id: yasm.c 2060 2008-04-12 01:18:02Z peter $");
 
 #include <ctype.h>
 #include <libyasm/compat-queue.h>
@@ -44,7 +44,7 @@
 #define PREPROC_BUF_SIZE    16384
 
 /*@null@*/ /*@only@*/ static char *obj_filename = NULL, *in_filename = NULL;
-/*@null@*/ /*@only@*/ static char *list_filename = NULL;
+/*@null@*/ /*@only@*/ static char *list_filename = NULL, *map_filename = NULL;
 /*@null@*/ /*@only@*/ static char *machine_name = NULL;
 static int special_options = 0;
 /*@null@*/ /*@dependent@*/ static yasm_arch *cur_arch = NULL;
@@ -91,6 +91,7 @@ static int opt_dbgfmt_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_listfmt_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_listfile_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_objfile_handler(char *cmd, /*@null@*/ char *param, int extra);
+static int opt_mapfile_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_machine_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_strict_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_warning_handler(char *cmd, /*@null@*/ char *param, int extra);
@@ -153,6 +154,8 @@ static opt_option options[] =
       N_("name of list-file output"), N_("listfile") },
     { 'o', "objfile", 1, opt_objfile_handler, 0,
       N_("name of object-file output"), N_("filename") },
+    { 0, "mapfile", 1, opt_mapfile_handler, 0,
+      N_("name of map-file output"), N_("filename") },
     { 'm', "machine", 1, opt_machine_handler, 0,
       N_("select machine (list with -m help)"), N_("machine") },
     { 0, "force-strict", 0, opt_strict_handler, 0,
@@ -191,7 +194,7 @@ static opt_option options[] =
 /*@observer@*/ static const char *version_msg[] = {
     PACKAGE_NAME " " PACKAGE_INTVER "." PACKAGE_BUILD,
     "Compiled on " __DATE__ ".",
-    "Copyright (c) 2001-2007 Peter Johnson and other Yasm developers.",
+    "Copyright (c) 2001-2008 Peter Johnson and other Yasm developers.",
     "Run yasm --license for licensing overview and summary."
 };
 
@@ -223,7 +226,7 @@ static int
 do_preproc_only(void)
 {
     yasm_linemap *linemap;
-    char *preproc_buf = yasm_xmalloc(PREPROC_BUF_SIZE);
+    char *preproc_buf;
     size_t got;
     const char *base_filename;
     FILE *out = NULL;
@@ -257,14 +260,12 @@ do_preproc_only(void)
     } else {
         /* Open output (object) file */
         out = open_file(obj_filename, "wt");
-        if (!out) {
-            yasm_xfree(preproc_buf);
+        if (!out)
             return EXIT_FAILURE;
-        }
     }
 
     /* Pre-process until done */
-    cur_preproc = yasm_preproc_create(cur_preproc_module, in_filename,
+    cur_preproc = yasm_preproc_create(cur_preproc_module, in_filename, NULL,
                                       linemap, errwarns);
 
     apply_preproc_builtins();
@@ -273,6 +274,8 @@ do_preproc_only(void)
     if (generate_make_dependencies) {
         size_t totlen;
 
+        preproc_buf = yasm_xmalloc(PREPROC_BUF_SIZE);
+
         fprintf(stdout, "%s: %s", obj_filename, in_filename);
         totlen = strlen(obj_filename)+2+strlen(in_filename);
 
@@ -287,10 +290,13 @@ do_preproc_only(void)
             fwrite(preproc_buf, got, 1, stdout);
         }
         fputc('\n', stdout);
+        yasm_xfree(preproc_buf);
     } else {
-        while ((got = yasm_preproc_input(cur_preproc, preproc_buf,
-                                         PREPROC_BUF_SIZE)) != 0)
-            fwrite(preproc_buf, got, 1, out);
+        while ((preproc_buf = yasm_preproc_get_line(cur_preproc)) != NULL) {
+            fputs(preproc_buf, out);
+            fputc('\n', out);
+            yasm_xfree(preproc_buf);
+        }
     }
 
     if (out != stdout)
@@ -301,7 +307,6 @@ do_preproc_only(void)
                                  print_yasm_error, print_yasm_warning);
         if (out != stdout)
             remove(obj_filename);
-        yasm_xfree(preproc_buf);
         yasm_linemap_destroy(linemap);
         yasm_errwarns_destroy(errwarns);
         cleanup(NULL);
@@ -310,7 +315,6 @@ do_preproc_only(void)
 
     yasm_errwarns_output_all(errwarns, linemap, warning_error,
                              print_yasm_error, print_yasm_warning);
-    yasm_xfree(preproc_buf);
     yasm_linemap_destroy(linemap);
     yasm_errwarns_destroy(errwarns);
     cleanup(NULL);
@@ -422,7 +426,7 @@ do_assemble(void)
     }
 
     cur_preproc = yasm_preproc_create(cur_preproc_module, in_filename,
-                                      linemap, errwarns);
+                                      object->symtab, linemap, errwarns);
 
     apply_preproc_builtins();
     apply_preproc_saved_options();
@@ -435,6 +439,32 @@ do_assemble(void)
 
     yasm_arch_set_var(cur_arch, "force_strict", force_strict);
 
+    /* Try to enable the map file via a map NASM directive.  This is
+     * somewhat of a hack.
+     */
+    if (map_filename) {
+        const yasm_directive *dir = &cur_objfmt_module->directives[0];
+        matched = 0;
+        for (; dir && dir->name; dir++) {
+            if (yasm__strcasecmp(dir->name, "map") == 0 &&
+                yasm__strcasecmp(dir->parser, "nasm") == 0) {
+                yasm_valparamhead vps;
+                yasm_valparam *vp;
+                matched = 1;
+                yasm_vps_initialize(&vps);
+                vp = yasm_vp_create_string(NULL, yasm__xstrdup(map_filename));
+                yasm_vps_append(&vps, vp);
+                dir->handler(object, &vps, NULL, 0);
+                yasm_vps_delete(&vps);
+            }
+        }
+        if (!matched) {
+            print_error(
+                _("warning: object format `%s' does not support map files"),
+                cur_objfmt_module->keyword);
+        }
+    }
+
     /* Parse! */
     cur_parser_module->do_parse(object, cur_preproc, list_filename != NULL,
                                 linemap, errwarns);
@@ -717,6 +747,8 @@ cleanup(yasm_object *object)
             yasm_xfree(obj_filename);
         if (list_filename)
             yasm_xfree(list_filename);
+        if (map_filename)
+            yasm_xfree(map_filename);
         if (machine_name)
             yasm_xfree(machine_name);
         if (objfmt_keyword)
@@ -922,6 +954,22 @@ opt_objfile_handler(/*@unused@*/ char *cmd, char *param,
     return 0;
 }
 
+static int
+opt_mapfile_handler(/*@unused@*/ char *cmd, char *param,
+                    /*@unused@*/ int extra)
+{
+    if (map_filename) {
+        print_error(
+            _("warning: can output to only one map file, last specified used"));
+        yasm_xfree(map_filename);
+    }
+
+    assert(param != NULL);
+    map_filename = yasm__xstrdup(param);
+
+    return 0;
+}
+
 static int
 opt_machine_handler(/*@unused@*/ char *cmd, char *param,
                     /*@unused@*/ int extra)
@@ -1224,16 +1272,17 @@ print_yasm_error(const char *filename, unsigned long line, const char *msg,
                  const char *xref_msg)
 {
     if (line)
-        fprintf(errfile, fmt[ewmsg_style], filename, line, "", msg);
+        fprintf(errfile, fmt[ewmsg_style], filename, line, _("error: "), msg);
     else
-        fprintf(errfile, fmt_noline[ewmsg_style], filename, "", msg);
+        fprintf(errfile, fmt_noline[ewmsg_style], filename, _("error: "), msg);
 
     if (xref_fn && xref_msg) {
         if (xref_line)
-            fprintf(errfile, fmt[ewmsg_style], xref_fn, xref_line, "",
+            fprintf(errfile, fmt[ewmsg_style], xref_fn, xref_line, _("error: "),
                     xref_msg);
         else
-            fprintf(errfile, fmt_noline[ewmsg_style], xref_fn, "", xref_msg);
+            fprintf(errfile, fmt_noline[ewmsg_style], xref_fn, _("error: "),
+                    xref_msg);
     }
 }
 
