@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include "util.h"
-/*@unused@*/ RCSID("$Id: intnum.c 1980 2007-09-21 17:56:05Z peter $");
+/*@unused@*/ RCSID("$Id: intnum.c 2051 2008-04-11 09:29:58Z peter $");
 
 #include <ctype.h>
 #include <limits.h>
@@ -39,7 +39,7 @@
 
 
 /* "Native" "word" size for intnum calculations. */
-#define BITVECT_NATIVE_SIZE     128
+#define BITVECT_NATIVE_SIZE     256
 
 struct yasm_intnum {
     union val {
@@ -562,12 +562,56 @@ yasm_intnum_calc(yasm_intnum *acc, yasm_expr_op op, yasm_intnum *operand)
 }
 /*@=nullderef =nullpass =branchstate@*/
 
+int
+yasm_intnum_compare(const yasm_intnum *intn1, const yasm_intnum *intn2)
+{
+    wordptr op1, op2;
+
+    if (intn1->type == INTNUM_L && intn2->type == INTNUM_L) {
+        if (intn1->val.l < intn2->val.l)
+            return -1;
+        if (intn1->val.l > intn2->val.l)
+            return 1;
+        return 0;
+    }
+
+    op1 = intnum_tobv(op1static, intn1);
+    op2 = intnum_tobv(op2static, intn2);
+    return BitVector_Compare(op1, op2);
+}
+
 void
 yasm_intnum_zero(yasm_intnum *intn)
 {
     yasm_intnum_set_int(intn, 0);
 }
 
+void
+yasm_intnum_set(yasm_intnum *intn, const yasm_intnum *val)
+{
+    if (intn->type == val->type) {
+        switch (val->type) {
+            case INTNUM_L:
+                intn->val.l = val->val.l;
+                break;
+            case INTNUM_BV:
+                BitVector_Copy(intn->val.bv, val->val.bv);
+                break;
+        }
+    } else {
+        switch (val->type) {
+            case INTNUM_L:
+                BitVector_Destroy(intn->val.bv);
+                intn->val.l = val->val.l;
+                break;
+            case INTNUM_BV:
+                intn->val.bv = BitVector_Clone(val->val.bv);
+                break;
+        }
+        intn->type = val->type;
+    }
+}
+
 void
 yasm_intnum_set_uint(yasm_intnum *intn, unsigned long val)
 {
