@@ -3,11 +3,11 @@
  * \brief YASM value interface.
  *
  * \rcs
- * $Id: value.h 1740 2007-01-21 22:01:34Z peter $
+ * $Id: value.h 1827 2007-04-22 05:09:49Z peter $
  * \endrcs
  *
  * \license
- *  Copyright (C) 2006  Peter Johnson
+ *  Copyright (C) 2006-2007  Peter Johnson
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -41,52 +41,52 @@
  * parsing simply to ensure all fields of the value are initialized; after
  * the parse is complete, yasm_value_extract() should be called to finalize
  * the value.  The value defaults to unsigned.
- * \param value	    value to be initialized
- * \param e	    expression (kept)
- * \param size	    value size (in bits)
+ * \param value     value to be initialized
+ * \param e         expression (kept)
+ * \param size      value size (in bits)
  */
 void yasm_value_initialize(/*@out@*/ yasm_value *value,
-			   /*@null@*/ /*@kept@*/ yasm_expr *e,
-			   unsigned int size);
+                           /*@null@*/ /*@kept@*/ yasm_expr *e,
+                           unsigned int size);
 
 /** Initialize a #yasm_value with just a symrec.  No processing is performed,
  * the symrec is simply stuck into value.rel and the other fields are
  * initialized.
- * \param value	    value to be initialized
- * \param sym	    symrec
- * \param size	    value size (in bits)
+ * \param value     value to be initialized
+ * \param sym       symrec
+ * \param size      value size (in bits)
  */
 void yasm_value_init_sym(/*@out@*/ yasm_value *value,
-			 /*@null@*/ yasm_symrec *sym, unsigned int size);
+                         /*@null@*/ yasm_symrec *sym, unsigned int size);
 
 /** Initialize a #yasm_value as a copy of another yasm_value.  Any expressions
  * within orig are copied, so it's safe to delete the copy.
- * \param value	    value (copy to create)
- * \param orig	    original value
+ * \param value     value (copy to create)
+ * \param orig      original value
  */
 void yasm_value_init_copy(yasm_value *value, const yasm_value *orig);
 
 /** Frees any memory inside value; does not free value itself.
- * \param value	    value
+ * \param value     value
  */
 void yasm_value_delete(yasm_value *value);
 
 /** Set a value to be relative to the current assembly position rather than
  * relative to the section start.
- * \param value	    value
- * \param bc	    bytecode containing value
+ * \param value     value
+ * \param bc        bytecode containing value
  * \param ip_rel    if nonzero, indicates IP-relative data relocation,
- *		    sometimes used to generate special relocations
+ *                  sometimes used to generate special relocations
  * \note If value is just an absolute value, will get an absolute symrec to
  *       reference to (via bc's symbol table).
  */
 void yasm_value_set_curpos_rel(yasm_value *value, yasm_bytecode *bc,
-			       unsigned int ip_rel);
+                               unsigned int ip_rel);
 
 /** Perform yasm_value_finalize_expr() on a value that already exists from
  * being initialized with yasm_value_initialize().
- * \param value		value
- * \param precbc	previous bytecode to bytecode containing value
+ * \param value         value
+ * \param precbc        previous bytecode to bytecode containing value
  * \return Nonzero if value could not be split.
  */
 int yasm_value_finalize(yasm_value *value, /*@null@*/ yasm_bytecode *precbc);
@@ -98,30 +98,30 @@ int yasm_value_finalize(yasm_value *value, /*@null@*/ yasm_bytecode *precbc);
  * and yasm_value_finalize().  First expands references to symrecs in
  * absolute sections by expanding with the absolute section start plus the
  * symrec offset within the absolute section.
- * \param value		value to store split portions into
- * \param e		expression input
- * \param precbc	previous bytecode to bytecode containing expression
- * \param size		value size (in bits)
+ * \param value         value to store split portions into
+ * \param e             expression input
+ * \param precbc        previous bytecode to bytecode containing expression
+ * \param size          value size (in bits)
  * \return Nonzero if the expr could not be split into a value for some
  *         reason (e.g. the relative portion was not added, but multiplied,
  *         etc).
- * \caution Do not use e after this call.  Even if an error is returned, e
+ * \warning Do not use e after this call.  Even if an error is returned, e
  *          is stored into value.
  * \note This should only be called after the parse is complete.  Calling
  *       before the parse is complete will usually result in an error return.
  */
 int yasm_value_finalize_expr(/*@out@*/ yasm_value *value,
-			     /*@null@*/ /*@kept@*/ yasm_expr *e,
-			     /*@null@*/ yasm_bytecode *precbc,
-			     unsigned int size);
+                             /*@null@*/ /*@kept@*/ yasm_expr *e,
+                             /*@null@*/ yasm_bytecode *precbc,
+                             unsigned int size);
 
 /** Get value if absolute or PC-relative section-local relative.  Returns NULL
  * otherwise.
- * \param value		value
- * \param bc		current bytecode (for PC-relative calculation); if
- *			NULL, NULL is returned for PC-relative values.
- * \param calc_bc_dist	if nonzero, calculates bytecode distances in absolute
- *			portion of value
+ * \param value         value
+ * \param bc            current bytecode (for PC-relative calculation); if
+ *                      NULL, NULL is returned for PC-relative values.
+ * \param calc_bc_dist  if nonzero, calculates bytecode distances in absolute
+ *                      portion of value
  * \note Adds in value.rel (correctly) if PC-relative and in the same section
  *       as bc (and there is no WRT or SEG).
  * \return Intnum if can be resolved to integer value, otherwise NULL.
@@ -132,15 +132,15 @@ int yasm_value_finalize_expr(/*@out@*/ yasm_value *value,
 /** Output value if constant or PC-relative section-local.  This should be
  * used from objfmt yasm_output_value_func() functions.
  * functions.
- * \param value		value
- * \param buf		buffer for byte representation
- * \param destsize	destination size (in bytes)
- * \param bc		current bytecode (usually passed into higher-level
- *			calling function)
- * \param warn		enables standard warnings: zero for none;
- *			nonzero for overflow/underflow floating point and
- *			integer warnings
- * \param arch		architecture
+ * \param value         value
+ * \param buf           buffer for byte representation
+ * \param destsize      destination size (in bytes)
+ * \param bc            current bytecode (usually passed into higher-level
+ *                      calling function)
+ * \param warn          enables standard warnings: zero for none;
+ *                      nonzero for overflow/underflow floating point and
+ *                      integer warnings
+ * \param arch          architecture
  * \note Adds in value.rel (correctly) if PC-relative and in the same section
  *       as bc (and there is no WRT or SEG); if this is not the desired
  *       behavior, e.g. a reloc is needed in this case, don't use this
@@ -153,9 +153,9 @@ int yasm_value_output_basic
      yasm_bytecode *bc, int warn, yasm_arch *arch);
 
 /** Print a value.  For debugging purposes.
- * \param value		value
- * \param indent_level	indentation level
- * \param f		file
+ * \param value         value
+ * \param indent_level  indentation level
+ * \param f             file
  */
 void yasm_value_print(const yasm_value *value, FILE *f, int indent_level);
 
