@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: x86expr.c 1984 2007-09-21 18:16:28Z peter $");
+/*@unused@*/ RCSID("$Id: x86expr.c 2016 2007-11-28 07:25:59Z peter $");
 
 #include <libyasm.h>
 
@@ -375,6 +375,11 @@ x86_expr_checkea_getregusage(yasm_expr **ep, /*@null@*/ int *indexreg,
                     *indexreg = regnum;
             }
             break;
+        case YASM_EXPR_SEGOFF:
+            /* No registers are allowed on either side. */
+            if (yasm_expr__contains(e, YASM_EXPR_REG))
+                return 1;
+            break;
         default:
             /* Should never get here! */
             yasm_internal_error(N_("unexpected expr op"));
