@@ -26,7 +26,7 @@
  */
 #include <ctype.h>
 #include <util.h>
-RCSID("$Id: x86id.c 1984 2007-09-21 18:16:28Z peter $");
+RCSID("$Id: x86id.c 2051 2008-04-11 09:29:58Z peter $");
 
 #include <libyasm.h>
 #include <libyasm/phash.h>
@@ -49,6 +49,7 @@ static const char *cpu_find_reverse(unsigned int cpu0, unsigned int cpu1,
 #define MOD_AdSizeR 8   /* Parameter replaces addrsize (jmp only) */
 #define MOD_DOpS64R 9   /* Parameter replaces default 64-bit opersize */
 #define MOD_Op1AddSp 10 /* Parameter is added as "spare" to opcode byte 2 */
+#define MOD_SetVEX  11  /* Parameter replaces internal VEX prefix value */
 
 /* GAS suffix flags for instructions */
 enum x86_gas_suffix_flags {
@@ -69,6 +70,16 @@ enum x86_gas_suffix_flags {
     WEAK = 1<<5             /* Relaxed operand mode for GAS */
 };
 
+/* Miscellaneous flag tests for instructions */
+enum x86_misc_flags {
+    /* These are tested against BITS==64. */
+    ONLY_64 = 1<<0,         /* Only available in 64-bit mode */
+    NOT_64 = 1<<1,          /* Not available (invalid) in 64-bit mode */
+    /* These are tested against whether the base instruction is an AVX one. */
+    ONLY_AVX = 1<<2,        /* Only available in AVX instruction */
+    NOT_AVX = 1<<3          /* Not available (invalid) in AVX instruction */
+};
+
 enum x86_operand_type {
     OPT_Imm = 0,        /* immediate */
     OPT_Reg = 1,        /* any general purpose or FPU register */
@@ -111,18 +122,18 @@ enum x86_operand_type {
 enum x86_operand_size {
     /* any size acceptable/no size spec acceptable (dep. on strict) */
     OPS_Any = 0,
-    /* 8/16/32/64 bits (from user or reg size) */
+    /* 8/16/32/64/80/128/256 bits (from user or reg size) */
     OPS_8 = 1,
     OPS_16 = 2,
     OPS_32 = 3,
     OPS_64 = 4,
-    /* 80/128 bits (from user) */
     OPS_80 = 5,
     OPS_128 = 6,
+    OPS_256 = 7,
     /* current BITS setting; when this is used the size matched
      * gets stored into the opersize as well.
      */
-    OPS_BITS = 7
+    OPS_BITS = 8
 };
 
 enum x86_operand_targetmod {
@@ -153,7 +164,18 @@ enum x86_operand_action {
     OPA_JmpFar = 10,
     /* ea operand only sets address size (no actual ea field) */
     OPA_AdSizeEA = 11,
-    OPA_DREX = 12   /* operand data goes into DREX "dest" field */
+    OPA_DREX = 12,  /* operand data goes into DREX "dest" field */
+    OPA_VEX = 13,   /* operand data goes into VEX "vvvv" field */
+    /* operand data goes into BOTH VEX "vvvv" field and ea field */
+    OPA_EAVEX = 14,
+    /* operand data goes into BOTH VEX "vvvv" field and spare field */
+    OPA_SpareVEX = 15,
+    /* operand data goes into upper 4 bits of immediate byte (VEX is4 field) */
+    OPA_VEXImmSrc = 16,
+    /* operand data goes into bottom 4 bits of immediate byte
+     * (currently only VEX imz2 field)
+     */
+    OPA_VEXImm = 17
 };
 
 enum x86_operand_post_action {
@@ -177,7 +199,7 @@ typedef struct x86_info_operand {
     unsigned int type:5;
 
     /* size (user-specified, or from register size) */
-    unsigned int size:3;
+    unsigned int size:4;
 
     /* size implicit or explicit ("strictness" of size matching on
      * non-registers -- registers are always strictly matched):
@@ -201,7 +223,7 @@ typedef struct x86_info_operand {
      * an ea field).  Naturally, only one of each of these may be contained
      * in the operands of a single insn_info structure.
      */
-    unsigned int action:4;
+    unsigned int action:5;
 
     /* Postponed actions: actions which can't be completed at
      * parse-time due to possibly dependent expressions.  For these, some
@@ -216,14 +238,17 @@ typedef struct x86_insn_info {
     /* GAS suffix flags */
     unsigned int gas_flags:8;      /* Enabled for these GAS suffixes */
 
+    /* Tests against BITS==64 and AVX */
+    unsigned int misc_flags:6;
+
     /* The CPU feature flags needed to execute this instruction.  This is OR'ed
      * with arch-specific data[2].  This combined value is compared with
      * cpu_enabled to see if all bits set here are set in cpu_enabled--if so,
      * the instruction is available on this CPU.
      */
-    unsigned int cpu0:8;
-    unsigned int cpu1:8;
-    unsigned int cpu2:8;
+    unsigned int cpu0:6;
+    unsigned int cpu1:6;
+    unsigned int cpu2:6;
 
     /* Opcode modifiers for variations of instruction.  As each modifier reads
      * its parameter in LSB->MSB order from the arch-specific data[1] from the
@@ -244,6 +269,14 @@ typedef struct x86_insn_info {
      * mode, they're treated like normal prefixes (e.g. the REX prefix needs
      * to be *after* the F2/F3/66 "prefix").
      * (0=no special prefix)
+     * 0xC0 - 0xCF indicate a VEX prefix, with the four LSBs holding "WLpp":
+     *  W: VEX.W field (meaning depends on opcode)
+     *  L: 0=128-bit, 1=256-bit
+     *  pp: SIMD prefix designation:
+     *      00: None
+     *      01: 66
+     *      10: F3
+     *      11: F2
      */
     unsigned char special_prefix;
 
@@ -296,6 +329,9 @@ typedef struct x86_id_insn {
     /* Suffix flags */
     unsigned int suffix:8;
 
+    /* Tests against BITS==64 and AVX */
+    unsigned int misc_flags:6;
+
     /* Parser enabled at the time of parsing the instruction */
     unsigned int parser:2;
 
@@ -360,7 +396,6 @@ x86_finalize_jmpfar(yasm_bytecode *bc, yasm_bytecode *prev_bc,
     unsigned int mode_bits = id_insn->mode_bits;
     x86_jmpfar *jmpfar;
     yasm_insn_operand *op;
-    /*@only@*/ yasm_expr *segment;
 
     jmpfar = yasm_xmalloc(sizeof(x86_jmpfar));
     x86_finalize_common(&jmpfar->common, info, mode_bits);
@@ -368,13 +403,9 @@ x86_finalize_jmpfar(yasm_bytecode *bc, yasm_bytecode *prev_bc,
 
     op = yasm_insn_ops_first(&id_insn->insn);
 
-    if (op->type == YASM_INSN__OPERAND_IMM &&
-        yasm_expr_is_op(op->data.val, YASM_EXPR_SEGOFF)) {
-        /* SEG:OFF expression; split it. */
-        segment = yasm_expr_extract_segoff(&op->data.val);
-        if (!segment)
-            yasm_internal_error(N_("didn't get SEG:OFF expression in jmpfar"));
-        if (yasm_value_finalize_expr(&jmpfar->segment, segment, prev_bc, 16))
+    if (op->type == YASM_INSN__OPERAND_IMM && op->seg) {
+        /* SEG:OFF */
+        if (yasm_value_finalize_expr(&jmpfar->segment, op->seg, prev_bc, 16))
             yasm_error_set(YASM_ERROR_TOO_COMPLEX,
                            N_("jump target segment too complex"));
         if (yasm_value_finalize_expr(&jmpfar->offset, op->data.val, prev_bc,
@@ -383,13 +414,13 @@ x86_finalize_jmpfar(yasm_bytecode *bc, yasm_bytecode *prev_bc,
                            N_("jump target offset too complex"));
     } else if (op->targetmod == X86_FAR) {
         /* "FAR imm" target needs to become "seg imm:imm". */
-        if (yasm_value_finalize_expr(&jmpfar->offset,
-                                     yasm_expr_copy(op->data.val), prev_bc, 0)
-            || yasm_value_finalize_expr(&jmpfar->segment, op->data.val,
-                                        prev_bc, 16))
+        yasm_expr *e = yasm_expr_create_branch(YASM_EXPR_SEG,
+                                               yasm_expr_copy(op->data.val),
+                                               op->data.val->line);
+        if (yasm_value_finalize_expr(&jmpfar->offset, op->data.val, prev_bc, 0)
+            || yasm_value_finalize_expr(&jmpfar->segment, e, prev_bc, 16))
             yasm_error_set(YASM_ERROR_TOO_COMPLEX,
                            N_("jump target expression too complex"));
-        jmpfar->segment.seg_of = 1;
     } else
         yasm_internal_error(N_("didn't get FAR expression in jmpfar"));
 
@@ -416,7 +447,8 @@ x86_finalize_jmp(yasm_bytecode *bc, yasm_bytecode *prev_bc,
     unsigned int mode_bits = id_insn->mode_bits;
     /*unsigned char suffix = id_insn->suffix;*/
     yasm_insn_operand *op;
-    static const unsigned char size_lookup[] = {0, 8, 16, 32, 64, 80, 128, 0};
+    static const unsigned char size_lookup[] =
+        {0, 8, 16, 32, 64, 80, 128, 0, 0};  /* 256 not needed */
     unsigned int i;
 
     /* We know the target is in operand 0, but sanity check for Imm. */
@@ -453,7 +485,7 @@ x86_finalize_jmp(yasm_bytecode *bc, yasm_bytecode *prev_bc,
             size_lookup[insn_operands[jinfo->operands_index+1].size];
 
     /* Check for address size override */
-    for (i=0; i<NELEMS(info->modifiers); i++) {
+    for (i=0; i<NELEMS(jinfo->modifiers); i++) {
         if (jinfo->modifiers[i] == MOD_AdSizeR)
             jmp->common.addrsize = mod_data[i];
     }
@@ -465,27 +497,15 @@ x86_finalize_jmp(yasm_bytecode *bc, yasm_bytecode *prev_bc,
     jmp->nearop.len = 0;
     for (; num_info>0 && (jmp->shortop.len == 0 || jmp->nearop.len == 0);
          num_info--, info++) {
-        unsigned int cpu0 = info->cpu0;
-        unsigned int cpu1 = info->cpu1;
-        unsigned int cpu2 = info->cpu2;
-
         /* Match CPU */
-        if (mode_bits != 64 &&
-            (cpu0 == CPU_64 || cpu1 == CPU_64 || cpu2 == CPU_64))
+        if (mode_bits != 64 && (info->misc_flags & ONLY_64))
             continue;
-        if (mode_bits == 64 &&
-            (cpu0 == CPU_Not64 || cpu1 == CPU_Not64 || cpu2 == CPU_Not64))
+        if (mode_bits == 64 && (info->misc_flags & NOT_64))
             continue;
 
-        if (cpu0 == CPU_64 || cpu0 == CPU_Not64)
-            cpu0 = CPU_Any;
-        if (cpu1 == CPU_64 || cpu1 == CPU_Not64)
-            cpu1 = CPU_Any;
-        if (cpu2 == CPU_64 || cpu2 == CPU_Not64)
-            cpu2 = CPU_Any;
-        if (!BitVector_bit_test(id_insn->cpu_enabled, cpu0) ||
-            !BitVector_bit_test(id_insn->cpu_enabled, cpu1) ||
-            !BitVector_bit_test(id_insn->cpu_enabled, cpu2))
+        if (!BitVector_bit_test(id_insn->cpu_enabled, info->cpu0) ||
+            !BitVector_bit_test(id_insn->cpu_enabled, info->cpu1) ||
+            !BitVector_bit_test(id_insn->cpu_enabled, info->cpu2))
             continue;
 
         if (info->num_operands == 0)
@@ -515,10 +535,10 @@ x86_finalize_jmp(yasm_bytecode *bc, yasm_bytecode *prev_bc,
         }
     }
 
-    if ((jmp->op_sel == JMP_SHORT_FORCED) && (jmp->nearop.len == 0))
+    if ((jmp->op_sel == JMP_SHORT_FORCED) && (jmp->shortop.len == 0))
         yasm_error_set(YASM_ERROR_TYPE,
                        N_("no SHORT form of that jump instruction exists"));
-    if ((jmp->op_sel == JMP_NEAR_FORCED) && (jmp->shortop.len == 0))
+    if ((jmp->op_sel == JMP_NEAR_FORCED) && (jmp->nearop.len == 0))
         yasm_error_set(YASM_ERROR_TYPE,
                        N_("no NEAR form of that jump instruction exists"));
 
@@ -529,7 +549,8 @@ x86_finalize_jmp(yasm_bytecode *bc, yasm_bytecode *prev_bc,
             jmp->op_sel = JMP_NEAR_FORCED;
     }
 
-    yasm_x86__bc_apply_prefixes((x86_common *)jmp, NULL, info->def_opersize_64,
+    yasm_x86__bc_apply_prefixes((x86_common *)jmp, NULL,
+                                jinfo->def_opersize_64,
                                 id_insn->insn.num_prefixes,
                                 id_insn->insn.prefixes);
 
@@ -557,37 +578,34 @@ x86_find_match(x86_id_insn *id_insn, yasm_insn_operand **ops,
         yasm_insn_operand *op, **use_ops;
         const x86_info_operand *info_ops =
             &insn_operands[info->operands_index];
-        unsigned int cpu0 = info->cpu0;
-        unsigned int cpu1 = info->cpu1;
-        unsigned int cpu2 = info->cpu2;
         unsigned int gas_flags = info->gas_flags;
+        unsigned int misc_flags = info->misc_flags;
         unsigned int size;
         int mismatch = 0;
         int i;
 
         /* Match CPU */
-        if (mode_bits != 64 &&
-            (cpu0 == CPU_64 || cpu1 == CPU_64 || cpu2 == CPU_64))
+        if (mode_bits != 64 && (misc_flags & ONLY_64))
             continue;
-        if (mode_bits == 64 &&
-            (cpu0 == CPU_Not64 || cpu1 == CPU_Not64 || cpu2 == CPU_Not64))
+        if (mode_bits == 64 && (misc_flags & NOT_64))
             continue;
 
-        if (cpu0 == CPU_64 || cpu0 == CPU_Not64)
-            cpu0 = CPU_Any;
-        if (cpu1 == CPU_64 || cpu1 == CPU_Not64)
-            cpu1 = CPU_Any;
-        if (cpu2 == CPU_64 || cpu2 == CPU_Not64)
-            cpu2 = CPU_Any;
-        if (bypass != 8 && (!BitVector_bit_test(id_insn->cpu_enabled, cpu0) ||
-                            !BitVector_bit_test(id_insn->cpu_enabled, cpu1) ||
-                            !BitVector_bit_test(id_insn->cpu_enabled, cpu2)))
+        if (bypass != 8 &&
+            (!BitVector_bit_test(id_insn->cpu_enabled, info->cpu0) ||
+             !BitVector_bit_test(id_insn->cpu_enabled, info->cpu1) ||
+             !BitVector_bit_test(id_insn->cpu_enabled, info->cpu2)))
             continue;
 
         /* Match # of operands */
         if (id_insn->insn.num_operands != info->num_operands)
             continue;
 
+        /* Match AVX */
+        if (!(id_insn->misc_flags & ONLY_AVX) && (misc_flags & ONLY_AVX))
+            continue;
+        if ((id_insn->misc_flags & ONLY_AVX) && (misc_flags & NOT_AVX))
+            continue;
+
         /* Match parser mode */
         if ((gas_flags & GAS_ONLY) && id_insn->parser != X86_PARSER_GAS)
             continue;
@@ -656,6 +674,7 @@ x86_find_match(x86_id_insn *id_insn, yasm_insn_operand **ops,
                         switch ((x86_expritem_reg_size)(op->data.reg&~0xFUL)) {
                             case X86_MMXREG:
                             case X86_XMMREG:
+                            case X86_YMMREG:
                                 break;
                             default:
                                 mismatch = 1;
@@ -784,8 +803,7 @@ x86_find_match(x86_id_insn *id_insn, yasm_insn_operand **ops,
                     break;
                 case OPT_ImmNotSegOff:
                     if (op->type != YASM_INSN__OPERAND_IMM ||
-                        op->targetmod != 0 ||
-                        yasm_expr_is_op(op->data.val, YASM_EXPR_SEGOFF))
+                        op->targetmod != 0 || op->seg)
                         mismatch = 1;
                     break;
                 case OPT_XMM0:
@@ -838,13 +856,13 @@ x86_find_match(x86_id_insn *id_insn, yasm_insn_operand **ops,
                 if (op->type == YASM_INSN__OPERAND_REG && op->size == 0) {
                     /* Register size must exactly match */
                     if ((bypass == 4 && i == 0) || (bypass == 5 && i == 1)
-                        || (bypass == 6 && i == 3))
+                        || (bypass == 6 && i == 2))
                         ;
                     else if (yasm_x86__get_reg_size(op->data.reg) != size)
                         mismatch = 1;
                 } else {
                     if ((bypass == 1 && i == 0) || (bypass == 2 && i == 1)
-                        || (bypass == 3 && i == 3))
+                        || (bypass == 3 && i == 2))
                         ;
                     else if (info_ops[i].relaxed) {
                         /* Relaxed checking */
@@ -981,29 +999,30 @@ x86_id_insn_finalize(yasm_bytecode *bc, yasm_bytecode *prev_bc)
     const x86_insn_info *info = id_insn->group;
     unsigned int mode_bits = id_insn->mode_bits;
     unsigned char *mod_data = id_insn->mod_data;
-    yasm_insn_operand *op, *ops[4], *rev_ops[4];
+    yasm_insn_operand *op, *ops[5], *rev_ops[5];
     /*@null@*/ yasm_expr *imm;
     unsigned char im_len;
     unsigned char im_sign;
     unsigned char spare;
     unsigned char drex;
     unsigned char *pdrex;
+    unsigned char vexdata, vexreg;
     unsigned int i;
-    unsigned int size_lookup[] = {0, 8, 16, 32, 64, 80, 128, 0};
+    unsigned int size_lookup[] = {0, 8, 16, 32, 64, 80, 128, 256, 0};
     unsigned long do_postop = 0;
 
-    size_lookup[7] = mode_bits;
+    size_lookup[OPS_BITS] = mode_bits;
 
     yasm_insn_finalize(&id_insn->insn);
 
     /* Build local array of operands from list, since we know we have a max
-     * of 4 operands.
+     * of 5 operands.
      */
-    if (id_insn->insn.num_operands > 4) {
+    if (id_insn->insn.num_operands > 5) {
         yasm_error_set(YASM_ERROR_TYPE, N_("too many operands"));
         return;
     }
-    ops[0] = ops[1] = ops[2] = ops[3] = NULL;
+    ops[0] = ops[1] = ops[2] = ops[3] = ops[4] = NULL;
     for (i = 0, op = yasm_insn_ops_first(&id_insn->insn);
          op && i < id_insn->insn.num_operands;
          op = yasm_insn_op_next(op), i++)
@@ -1013,7 +1032,7 @@ x86_id_insn_finalize(yasm_bytecode *bc, yasm_bytecode *prev_bc)
      * as most GAS instructions have reversed operands from Intel style.
      */
     if (id_insn->parser == X86_PARSER_GAS) {
-        rev_ops[0] = rev_ops[1] = rev_ops[2] = rev_ops[3] = NULL;
+        rev_ops[0] = rev_ops[1] = rev_ops[2] = rev_ops[3] = rev_ops[4] = NULL;
         for (i = id_insn->insn.num_operands-1,
              op = yasm_insn_ops_first(&id_insn->insn);
              op; op = yasm_insn_op_next(op), i--)
@@ -1080,12 +1099,22 @@ x86_id_insn_finalize(yasm_bytecode *bc, yasm_bytecode *prev_bc)
     insn->special_prefix = info->special_prefix;
     spare = info->spare;
     drex = info->drex_oc0 & DREX_OC0_MASK;
+    vexdata = 0;
+    vexreg = 0;
     im_len = 0;
     im_sign = 0;
     insn->postop = X86_POSTOP_NONE;
     insn->rex = 0;
     pdrex = (info->drex_oc0 & NEED_DREX_MASK) ? &drex : NULL;
 
+    /* Move VEX data (stored in special prefix) to separate location to
+     * allow overriding of special prefix by modifiers.
+     */
+    if ((insn->special_prefix & 0xF0) == 0xC0) {
+        vexdata = insn->special_prefix;
+        insn->special_prefix = 0;
+    }
+
     /* Apply modifiers */
     for (i=0; i<NELEMS(info->modifiers); i++) {
         switch (info->modifiers[i]) {
@@ -1120,6 +1149,9 @@ x86_id_insn_finalize(yasm_bytecode *bc, yasm_bytecode *prev_bc)
             case MOD_Op1AddSp:
                 insn->opcode.opcode[1] += mod_data[i]<<3;
                 break;
+            case MOD_SetVEX:
+                vexdata = mod_data[i];
+                break;
             default:
                 break;
         }
@@ -1172,6 +1204,9 @@ x86_id_insn_finalize(yasm_bytecode *bc, yasm_bytecode *prev_bc)
                             yasm_internal_error(
                                 N_("invalid operand conversion"));
                         case YASM_INSN__OPERAND_MEMORY:
+                            if (op->seg)
+                                yasm_error_set(YASM_ERROR_VALUE,
+                                    N_("invalid segment in effective address"));
                             insn->x86_ea = (x86_effaddr *)op->data.ea;
                             if (info_ops[i].type == OPT_MemOffs)
                                 /* Special-case for MOV MemOffs instruction */
@@ -1195,7 +1230,19 @@ x86_id_insn_finalize(yasm_bytecode *bc, yasm_bytecode *prev_bc)
                             break;
                     }
                     break;
+                case OPA_EAVEX:
+                    if (op->type != YASM_INSN__OPERAND_REG)
+                        yasm_internal_error(N_("invalid operand conversion"));
+                    insn->x86_ea =
+                        yasm_x86__ea_create_reg(insn->x86_ea,
+                            (unsigned long)op->data.reg, &insn->rex, pdrex,
+                            mode_bits);
+                    vexreg = op->data.reg & 0xF;
+                    break;
                 case OPA_Imm:
+                    if (op->seg)
+                        yasm_error_set(YASM_ERROR_VALUE,
+                                       N_("immediate does not support segment"));
                     if (op->type == YASM_INSN__OPERAND_IMM) {
                         imm = op->data.val;
                         im_len = size_lookup[info_ops[i].size];
@@ -1203,6 +1250,9 @@ x86_id_insn_finalize(yasm_bytecode *bc, yasm_bytecode *prev_bc)
                         yasm_internal_error(N_("invalid operand conversion"));
                     break;
                 case OPA_SImm:
+                    if (op->seg)
+                        yasm_error_set(YASM_ERROR_VALUE,
+                                       N_("immediate does not support segment"));
                     if (op->type == YASM_INSN__OPERAND_IMM) {
                         imm = op->data.val;
                         im_len = size_lookup[info_ops[i].size];
@@ -1220,6 +1270,14 @@ x86_id_insn_finalize(yasm_bytecode *bc, yasm_bytecode *prev_bc)
                     } else
                         yasm_internal_error(N_("invalid operand conversion"));
                     break;
+                case OPA_SpareVEX:
+                    if (op->type != YASM_INSN__OPERAND_REG)
+                        yasm_internal_error(N_("invalid operand conversion"));
+                    if (yasm_x86__set_rex_from_reg(&insn->rex, pdrex,
+                            &spare, op->data.reg, mode_bits, X86_REX_R))
+                        return;
+                    vexreg = op->data.reg & 0xF;
+                    break;
                 case OPA_Op0Add:
                     if (op->type == YASM_INSN__OPERAND_REG) {
                         unsigned char opadd;
@@ -1282,8 +1340,64 @@ x86_id_insn_finalize(yasm_bytecode *bc, yasm_bytecode *prev_bc)
                     break;
                 }
                 case OPA_DREX:
-                    drex &= 0x0F;
-                    drex |= (op->data.reg << 4) & 0xF0;
+                    if (op->type == YASM_INSN__OPERAND_REG) {
+                        drex &= 0x0F;
+                        drex |= (op->data.reg << 4) & 0xF0;
+                    } else
+                        yasm_internal_error(N_("invalid operand conversion"));
+                    break;
+                case OPA_VEX:
+                    if (op->type != YASM_INSN__OPERAND_REG)
+                        yasm_internal_error(N_("invalid operand conversion"));
+                    vexreg = op->data.reg & 0xF;
+                    break;
+                case OPA_VEXImmSrc:
+                    if (op->type != YASM_INSN__OPERAND_REG)
+                        yasm_internal_error(N_("invalid operand conversion"));
+
+                    if (!imm) {
+                        imm = yasm_expr_create_ident(
+                            yasm_expr_int(
+                                yasm_intnum_create_uint((op->data.reg << 4)
+                                                        & 0xF0)),
+                            bc->line);
+                    } else {
+                        imm = yasm_expr_create(
+                            YASM_EXPR_OR,
+                            yasm_expr_expr(yasm_expr_create(
+                                YASM_EXPR_AND,
+                                yasm_expr_expr(imm),
+                                yasm_expr_int(yasm_intnum_create_uint(0x0F)),
+                                bc->line)),
+                            yasm_expr_int(
+                                yasm_intnum_create_uint((op->data.reg << 4)
+                                                        & 0xF0)),
+                            bc->line);
+                    }
+                    im_len = 8;
+                    break;
+                case OPA_VEXImm:
+                    if (op->type != YASM_INSN__OPERAND_IMM)
+                        yasm_internal_error(N_("invalid operand conversion"));
+
+                    if (!imm)
+                        imm = op->data.val;
+                    else {
+                        imm = yasm_expr_create(
+                            YASM_EXPR_OR,
+                            yasm_expr_expr(yasm_expr_create(
+                                YASM_EXPR_AND,
+                                yasm_expr_expr(op->data.val),
+                                yasm_expr_int(yasm_intnum_create_uint(0x0F)),
+                                bc->line)),
+                            yasm_expr_expr(yasm_expr_create(
+                                YASM_EXPR_AND,
+                                yasm_expr_expr(imm),
+                                yasm_expr_int(yasm_intnum_create_uint(0xF0)),
+                                bc->line)),
+                            bc->line);
+                    }
+                    im_len = 8;
                     break;
                 default:
                     yasm_internal_error(N_("unknown operand action"));
@@ -1342,10 +1456,9 @@ x86_id_insn_finalize(yasm_bytecode *bc, yasm_bytecode *prev_bc)
 
     if (imm) {
         insn->imm = yasm_xmalloc(sizeof(yasm_value));
-        if (yasm_value_finalize_expr(insn->imm, imm, prev_bc, 0))
+        if (yasm_value_finalize_expr(insn->imm, imm, prev_bc, im_len))
             yasm_error_set(YASM_ERROR_TOO_COMPLEX,
                            N_("immediate expression too complex"));
-        insn->imm->size = im_len;
         insn->imm->sign = im_sign;
     } else
         insn->imm = NULL;
@@ -1412,6 +1525,66 @@ x86_id_insn_finalize(yasm_bytecode *bc, yasm_bytecode *prev_bc)
             break;
     }
 
+    /* Convert to VEX prefixes if requested.
+     * To save space in the insn structure, the VEX prefix is written into
+     * special_prefix and the first 2 bytes of the instruction are set to
+     * the second two VEX bytes.  During calc_len() it may be shortened to
+     * one VEX byte (this can only be done after knowledge of REX value).
+     */
+    if (vexdata) {
+        unsigned char vex1 = 0xE0;  /* R=X=B=1, mmmmm=0 */
+        unsigned char vex2;
+        /* Look at the first bytes of the opcode to see what leading bytes
+         * to encode in the VEX mmmmm field.  Leave R=X=B=1 for now.
+         */
+        if (insn->opcode.opcode[0] != 0x0F)
+            yasm_internal_error(N_("first opcode byte of VEX must be 0x0F"));
+
+        if (insn->opcode.opcode[1] == 0x38)
+            vex1 |= 0x02;       /* implied 0x0F 0x38 */
+        else if (insn->opcode.opcode[1] == 0x3A)
+            vex1 |= 0x03;       /* implied 0x0F 0x3A */
+        else {
+            /* Originally a 0F-only opcode; move opcode byte back one position
+             * to make room for VEX prefix.
+             */
+            insn->opcode.opcode[2] = insn->opcode.opcode[1];
+            vex1 |= 0x01;       /* implied 0x0F */
+        }
+
+        /* Check for update of special prefix by modifiers */
+        if (insn->special_prefix != 0) {
+            vexdata &= ~0x03;
+            switch (insn->special_prefix) {
+                case 0x66:
+                    vexdata |= 0x01;
+                    break;
+                case 0xF3:
+                    vexdata |= 0x02;
+                    break;
+                case 0xF2:
+                    vexdata |= 0x03;
+                    break;
+                default:
+                    yasm_internal_error(N_("unrecognized special prefix"));
+            }
+        }
+
+        /* 2nd VEX byte is WvvvvLpp.
+         * W, L, pp come from vexdata
+         * vvvv comes from 1s complement of vexreg
+         */
+        vex2 = (((vexdata & 0x8) << 4) |                /* W */
+                ((15 - (vexreg & 0xF)) << 3) |          /* vvvv */
+                (vexdata & 0x7));                       /* Lpp */
+
+        /* Save to special_prefix and opcode */
+        insn->special_prefix = 0xC4;    /* VEX prefix */
+        insn->opcode.opcode[0] = vex1;
+        insn->opcode.opcode[1] = vex2;
+        insn->opcode.len = 3;   /* two prefix bytes and 1 opcode byte */
+    }
+
     x86_id_insn_clear_operands(id_insn);
 
     /* Transform the bytecode */
@@ -1425,7 +1598,7 @@ typedef struct insnprefix_parse_data {
     /* instruction parse group - NULL if prefix */
     /*@null@*/ const x86_insn_info *group;
 
-    /* For instruction, number of elements in group in lower 8 bits.
+    /* For instruction, number of elements in group.
      * For prefix, prefix type shifted right by 8.
      */
     unsigned int num_info:8;
@@ -1440,10 +1613,13 @@ typedef struct insnprefix_parse_data {
     unsigned int mod_data1:8;
     unsigned int mod_data2:8;
 
+    /* Tests against BITS==64 and AVX */
+    unsigned int misc_flags:6;
+
     /* CPU flags */
-    unsigned int cpu0:8;
-    unsigned int cpu1:8;
-    unsigned int cpu2:8;
+    unsigned int cpu0:6;
+    unsigned int cpu1:6;
+    unsigned int cpu2:6;
 } insnprefix_parse_data;
 
 /* Pull in all parse data */
@@ -1541,7 +1717,6 @@ yasm_x86__parse_check_insnprefix(yasm_arch *arch, const char *id,
 {
     yasm_arch_x86 *arch_x86 = (yasm_arch_x86 *)arch;
     /*@null@*/ const insnprefix_parse_data *pdata;
-    unsigned int cpu0, cpu1, cpu2;
     size_t i;
     static char lcaseid[16];
 
@@ -1567,22 +1742,17 @@ yasm_x86__parse_check_insnprefix(yasm_arch *arch, const char *id,
     if (!pdata)
         return YASM_ARCH_NOTINSNPREFIX;
 
-    cpu0 = pdata->cpu0;
-    cpu1 = pdata->cpu1;
-    cpu2 = pdata->cpu2;
-
     if (pdata->group) {
         x86_id_insn *id_insn;
         wordptr cpu_enabled = arch_x86->cpu_enables[arch_x86->active_cpu];
+        unsigned int cpu0, cpu1, cpu2;
 
-        if (arch_x86->mode_bits != 64 &&
-            (cpu0 == CPU_64 || cpu1 == CPU_64 || cpu2 == CPU_64)) {
+        if (arch_x86->mode_bits != 64 && (pdata->misc_flags & ONLY_64)) {
             yasm_warn_set(YASM_WARN_GENERAL,
                           N_("`%s' is an instruction in 64-bit mode"), id);
             return YASM_ARCH_NOTINSNPREFIX;
         }
-        if (arch_x86->mode_bits == 64 &&
-            (cpu0 == CPU_Not64 || cpu1 == CPU_Not64 || cpu2 == CPU_Not64)) {
+        if (arch_x86->mode_bits == 64 && (pdata->misc_flags & NOT_64)) {
             yasm_error_set(YASM_ERROR_GENERAL,
                            N_("`%s' invalid in 64-bit mode"), id);
             id_insn = yasm_xmalloc(sizeof(x86_id_insn));
@@ -1595,6 +1765,7 @@ yasm_x86__parse_check_insnprefix(yasm_arch *arch, const char *id,
             id_insn->num_info = NELEMS(not64_insn);
             id_insn->mode_bits = arch_x86->mode_bits;
             id_insn->suffix = 0;
+            id_insn->misc_flags = 0;
             id_insn->parser = arch_x86->parser;
             id_insn->force_strict = arch_x86->force_strict != 0;
             id_insn->default_rel = arch_x86->default_rel != 0;
@@ -1602,12 +1773,10 @@ yasm_x86__parse_check_insnprefix(yasm_arch *arch, const char *id,
             return YASM_ARCH_INSN;
         }
 
-        if (cpu0 == CPU_64 || cpu0 == CPU_Not64)
-            cpu0 = CPU_Any;
-        if (cpu1 == CPU_64 || cpu1 == CPU_Not64)
-            cpu1 = CPU_Any;
-        if (cpu2 == CPU_64 || cpu2 == CPU_Not64)
-            cpu2 = CPU_Any;
+        cpu0 = pdata->cpu0;
+        cpu1 = pdata->cpu1;
+        cpu2 = pdata->cpu2;
+
         if (!BitVector_bit_test(cpu_enabled, cpu0) ||
             !BitVector_bit_test(cpu_enabled, cpu1) ||
             !BitVector_bit_test(cpu_enabled, cpu2)) {
@@ -1627,6 +1796,7 @@ yasm_x86__parse_check_insnprefix(yasm_arch *arch, const char *id,
         id_insn->num_info = pdata->num_info;
         id_insn->mode_bits = arch_x86->mode_bits;
         id_insn->suffix = pdata->flags;
+        id_insn->misc_flags = pdata->misc_flags;
         id_insn->parser = arch_x86->parser;
         id_insn->force_strict = arch_x86->force_strict != 0;
         id_insn->default_rel = arch_x86->default_rel != 0;
@@ -1648,8 +1818,7 @@ yasm_x86__parse_check_insnprefix(yasm_arch *arch, const char *id,
             return YASM_ARCH_NOTINSNPREFIX;
         }
 
-        if (arch_x86->mode_bits != 64 &&
-            (cpu0 == CPU_64 || cpu1 == CPU_64 || cpu2 == CPU_64)) {
+        if (arch_x86->mode_bits != 64 && (pdata->misc_flags & ONLY_64)) {
             yasm_warn_set(YASM_WARN_GENERAL,
                           N_("`%s' is a prefix in 64-bit mode"), id);
             return YASM_ARCH_NOTINSNPREFIX;
@@ -1690,6 +1859,7 @@ yasm_x86__create_empty_insn(yasm_arch *arch, unsigned long line)
     id_insn->num_info = NELEMS(empty_insn);
     id_insn->mode_bits = arch_x86->mode_bits;
     id_insn->suffix = 0;
+    id_insn->misc_flags = 0;
     id_insn->parser = arch_x86->parser;
     id_insn->force_strict = arch_x86->force_strict != 0;
     id_insn->default_rel = arch_x86->default_rel != 0;
