@@ -28,7 +28,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: cv-symline.c 1974 2007-09-21 16:37:01Z peter $");
+/*@unused@*/ RCSID("$Id: cv-symline.c 2010 2007-11-14 08:33:32Z peter $");
 
 #include <libyasm.h>
 
@@ -569,7 +569,7 @@ yasm_cv__generate_symline(yasm_object *object, yasm_linemap *linemap,
     info.linemap = linemap;
     info.errwarns = errwarns;
     info.debug_symline =
-        yasm_object_get_general(object, ".debug$S", 0, 1, 0, 0, &new, 0);
+        yasm_object_get_general(object, ".debug$S", 1, 0, 0, &new, 0);
     info.num_lineinfos = 0;
     STAILQ_INIT(&info.cv8_lineinfos);
     info.cv8_cur_li = NULL;
