@@ -28,7 +28,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: cv-type.c 1974 2007-09-21 16:37:01Z peter $");
+/*@unused@*/ RCSID("$Id: cv-type.c 2010 2007-11-14 08:33:32Z peter $");
 
 #include <libyasm.h>
 
@@ -522,7 +522,7 @@ yasm_cv__generate_type(yasm_object *object)
     cv_type *type;
 
     debug_type =
-        yasm_object_get_general(object, ".debug$T", 0, 1, 0, 0, &new, 0);
+        yasm_object_get_general(object, ".debug$T", 1, 0, 0, &new, 0);
 
     /* Add label type */
     type = cv_type_create(indx++);
