@@ -25,14 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: dwarf2-info.c 1974 2007-09-21 16:37:01Z peter $");
-
-/* Need either unistd.h or direct.h (on Windows) to prototype getcwd() */
-#if defined(HAVE_UNISTD_H)
-#include <unistd.h>
-#elif defined(HAVE_DIRECT_H)
-#include <direct.h>
-#endif
+/*@unused@*/ RCSID("$Id: dwarf2-info.c 2010 2007-11-14 08:33:32Z peter $");
 
 #include <libyasm.h>
 
@@ -274,9 +267,9 @@ yasm_dwarf2__generate_info(yasm_object *object, yasm_section *debug_line,
     dwarf2_head *head;
     char *buf;
     yasm_section *debug_abbrev =
-        yasm_object_get_general(object, ".debug_abbrev", 0, 4, 0, 0, &new, 0);
+        yasm_object_get_general(object, ".debug_abbrev", 4, 0, 0, &new, 0);
     yasm_section *debug_info =
-        yasm_object_get_general(object, ".debug_info", 0, 4, 0, 0, &new, 0);
+        yasm_object_get_general(object, ".debug_info", 4, 0, 0, &new, 0);
 
     yasm_section_set_align(debug_abbrev, 0, 0);
     yasm_section_set_align(debug_info, 0, 0);
@@ -337,9 +330,9 @@ yasm_dwarf2__generate_info(yasm_object *object, yasm_section *debug_line,
 
     /* compile directory (current working directory) */
     abc->len += dwarf2_add_abbrev_attr(abbrev, DW_AT_comp_dir, DW_FORM_string);
-    buf = getcwd(NULL, 0);
+    buf = yasm__getcwd();
     dwarf2_append_str(debug_info, buf);
-    free(buf);
+    yasm_xfree(buf);
 
     /* producer - assembler name */
     abc->len += dwarf2_add_abbrev_attr(abbrev, DW_AT_producer, DW_FORM_string);
