@@ -28,7 +28,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: dwarf2-line.c 1974 2007-09-21 16:37:01Z peter $");
+/*@unused@*/ RCSID("$Id: dwarf2-line.c 2010 2007-11-14 08:33:32Z peter $");
 
 #include <libyasm.h>
 
@@ -645,8 +645,8 @@ yasm_dwarf2__generate_line(yasm_object *object, yasm_linemap *linemap,
     info.object = object;
     info.linemap = linemap;
     info.dbgfmt_dwarf2 = dbgfmt_dwarf2;
-    info.debug_line = yasm_object_get_general(object, ".debug_line", 0, 1, 0,
-                                              0, &new, 0);
+    info.debug_line = yasm_object_get_general(object, ".debug_line", 1, 0, 0,
+                                              &new, 0);
     last = yasm_section_bcs_last(info.debug_line);
 
     /* header */
