@@ -26,29 +26,39 @@
  */
 
 #include <util.h>
-/*@unused@*/ RCSID("$Id: elf-x86-amd64.c 1976 2007-09-21 16:48:25Z peter $");
+/*@unused@*/ RCSID("$Id: elf-x86-amd64.c 2040 2008-02-21 08:57:23Z peter $");
 
 #include <libyasm.h>
 #define YASM_OBJFMT_ELF_INTERNAL
 #include "elf.h"
 #include "elf-machine.h"
 
-enum ssym_index {
-    SSYM_GOTPCREL = 0,
-    SSYM_GOT,
-    SSYM_PLT
+static elf_machine_ssym elf_x86_amd64_ssyms[] = {
+    {"plt",         ELF_SSYM_SYM_RELATIVE,  R_X86_64_PLT32,     32},
+    {"gotpcrel",    ELF_SSYM_SYM_RELATIVE,  R_X86_64_GOTPCREL,  32},
+    {"tlsgd",       ELF_SSYM_SYM_RELATIVE|ELF_SSYM_THREAD_LOCAL,
+                    R_X86_64_TLSGD,     32},
+    {"tlsld",       ELF_SSYM_SYM_RELATIVE|ELF_SSYM_THREAD_LOCAL,
+                    R_X86_64_TLSLD,     32},
+    {"gottpoff",    ELF_SSYM_SYM_RELATIVE|ELF_SSYM_THREAD_LOCAL,
+                    R_X86_64_GOTTPOFF,  32},
+    {"tpoff",       ELF_SSYM_SYM_RELATIVE|ELF_SSYM_THREAD_LOCAL,
+                    R_X86_64_TPOFF32,   32},
+    {"dtpoff",      ELF_SSYM_SYM_RELATIVE|ELF_SSYM_THREAD_LOCAL,
+                    R_X86_64_DTPOFF32,  32},
+    {"got",         ELF_SSYM_SYM_RELATIVE,  R_X86_64_GOT32,     32}
 };
 
 static int
 elf_x86_amd64_accepts_reloc(size_t val, yasm_symrec *wrt, yasm_symrec **ssyms)
 {
     if (wrt) {
-        if ((wrt == ssyms[SSYM_GOTPCREL] && val == 32)
-            || (wrt == ssyms[SSYM_GOT] && val == 32)
-            || (wrt == ssyms[SSYM_PLT] && val == 32))
-            return 1;
-        else
-            return 0;
+        size_t i;
+        for (i=0; i<NELEMS(elf_x86_amd64_ssyms); i++) {
+            if (wrt == ssyms[i] && val == elf_x86_amd64_ssyms[i].size)
+                return 1;
+        }
+        return 0;
     }
     return (val&(val-1)) ? 0 : ((val & (8|16|32|64)) != 0);
 }
@@ -133,14 +143,23 @@ elf_x86_amd64_map_reloc_info_to_type(elf_reloc_entry *reloc,
                                      yasm_symrec **ssyms)
 {
     if (reloc->wrt) {
-        if (reloc->wrt == ssyms[SSYM_GOTPCREL] && reloc->valsize == 32)
-            return (unsigned char) R_X86_64_GOTPCREL;
-        else if (reloc->wrt == ssyms[SSYM_GOT] && reloc->valsize == 32)
-            return (unsigned char) R_X86_64_GOT32;
-        else if (reloc->wrt == ssyms[SSYM_PLT] && reloc->valsize == 32)
-            return (unsigned char) R_X86_64_PLT32;
-        else
-            yasm_internal_error(N_("Unsupported WRT"));
+        size_t i;
+        for (i=0; i<NELEMS(elf_x86_amd64_ssyms); i++) {
+            if (reloc->wrt == ssyms[i] &&
+                reloc->valsize == elf_x86_amd64_ssyms[i].size) {
+                /* Force TLS type; this is required by the linker. */
+                if (elf_x86_amd64_ssyms[i].sym_rel & ELF_SSYM_THREAD_LOCAL) {
+                    elf_symtab_entry *esym;
+
+                    esym = yasm_symrec_get_data(reloc->reloc.sym,
+                                                &elf_symrec_data);
+                    if (esym)
+                        esym->type = STT_TLS;
+                }
+                return (unsigned char) elf_x86_amd64_ssyms[i].reloc;
+            }
+        }
+        yasm_internal_error(N_("Unsupported WRT"));
     } else if (reloc->rtype_rel) {
         switch (reloc->valsize) {
             case 8: return (unsigned char) R_X86_64_PC8;
@@ -208,12 +227,6 @@ elf_x86_amd64_write_proghead(unsigned char **bufpp,
     *bufpp = bufp;
 }
 
-static elf_machine_ssym elf_x86_amd64_ssyms[] = {
-    {"..gotpcrel", ELF_SSYM_SYM_RELATIVE},
-    {"..got", ELF_SSYM_SYM_RELATIVE},
-    {"..plt", 0}
-};
-
 const elf_machine_handler
 elf_machine_handler_x86_amd64 = {
     "x86", "amd64", ".rela",
