@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: rdf-objfmt.c 1974 2007-09-21 16:37:01Z peter $");
+/*@unused@*/ RCSID("$Id: rdf-objfmt.c 2035 2008-02-09 03:35:07Z peter $");
 
 #include <libyasm.h>
 
@@ -839,7 +839,7 @@ rdf_objfmt_add_default_section(yasm_object *object)
     rdf_section_data *rsd;
     int isnew;
 
-    retval = yasm_object_get_general(object, ".text", 0, 0, 1, 0, &isnew, 0);
+    retval = yasm_object_get_general(object, ".text", 0, 1, 0, &isnew, 0);
     if (isnew) {
         rsd = rdf_objfmt_init_new_section(object, retval, ".text", 0);
         rsd->type = RDF_SECT_CODE;
@@ -946,7 +946,7 @@ rdf_objfmt_section_switch(yasm_object *object, yasm_valparamhead *valparams,
         yasm_intnum_destroy(data.reserved_intn);
     }
 
-    retval = yasm_object_get_general(object, sectname, 0, 0, 1,
+    retval = yasm_object_get_general(object, sectname, 0, 1,
                                      data.type == RDF_SECT_BSS, &isnew, line);
 
     if (isnew)
@@ -964,6 +964,13 @@ rdf_objfmt_section_switch(yasm_object *object, yasm_valparamhead *valparams,
     return retval;
 }
 
+static /*@observer@*/ /*@null@*/ yasm_symrec *
+rdf_objfmt_get_special_sym(yasm_object *object, const char *name,
+                           const char *parser)
+{
+    return NULL;
+}
+
 static void
 rdf_section_data_destroy(void *data)
 {
@@ -1063,5 +1070,6 @@ yasm_objfmt_module yasm_rdf_LTX_objfmt = {
     rdf_objfmt_output,
     rdf_objfmt_destroy,
     rdf_objfmt_add_default_section,
-    rdf_objfmt_section_switch
+    rdf_objfmt_section_switch,
+    rdf_objfmt_get_special_sym
 };
