@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: xdf-objfmt.c 1974 2007-09-21 16:37:01Z peter $");
+/*@unused@*/ RCSID("$Id: xdf-objfmt.c 2035 2008-02-09 03:35:07Z peter $");
 
 #include <libyasm.h>
 
@@ -634,7 +634,7 @@ xdf_objfmt_add_default_section(yasm_object *object)
     xdf_section_data *xsd;
     int isnew;
 
-    retval = yasm_object_get_general(object, ".text", 0, 0, 1, 0, &isnew, 0);
+    retval = yasm_object_get_general(object, ".text", 0, 1, 0, &isnew, 0);
     if (isnew) {
         xsd = xdf_objfmt_init_new_section(object, retval, ".text", 0);
         yasm_section_set_default(retval, 1);
@@ -745,7 +745,7 @@ xdf_objfmt_section_switch(yasm_object *object, yasm_valparamhead *valparams,
         }
     }
 
-    retval = yasm_object_get_general(object, sectname, 0, align, 1, resonly,
+    retval = yasm_object_get_general(object, sectname, align, 1, resonly,
                                      &isnew, line);
 
     if (isnew)
@@ -773,6 +773,13 @@ xdf_objfmt_section_switch(yasm_object *object, yasm_valparamhead *valparams,
     return retval;
 }
 
+static /*@observer@*/ /*@null@*/ yasm_symrec *
+xdf_objfmt_get_special_sym(yasm_object *object, const char *name,
+                           const char *parser)
+{
+    return NULL;
+}
+
 static void
 xdf_section_data_destroy(void *data)
 {
@@ -836,5 +843,6 @@ yasm_objfmt_module yasm_xdf_LTX_objfmt = {
     xdf_objfmt_output,
     xdf_objfmt_destroy,
     xdf_objfmt_add_default_section,
-    xdf_objfmt_section_switch
+    xdf_objfmt_section_switch,
+    xdf_objfmt_get_special_sym
 };
